<?php

/**
 * @file
 * Contains Drupal\instapage\Form\PagesForm.
 */

namespace Drupal\instapage\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Class PagesForm.
 *
 * @package Drupal\instapage\Form
 */
class PagesForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'instapage.pages',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'instapage_pages_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    global $base_url;
    $config = $this->config('instapage.settings');

    $instapage_user_id = $config->get('instapage_user_id');
    $plugin_hash = $config->get('instapage_plugin_hash');

    if (!$instapage_user_id && !$plugin_hash) {
      $form['notice']['#markup'] = '<p>' . t('You don\'t have the Instapage account setup yet.') . '</p>';
      $form['notice']['#markup'] .= '<p>' . t('Please connect your account <a href="@link">here</a>.', array('@link' => Url::fromRoute('instapage.settings')->toString())) . '</p>';
      return $form;
    }

    $data = array(
      'user_id' => $instapage_user_id,
      'plugin_hash' => $plugin_hash,
    );

    $result = instapage_api_call('my-pages/drupal', $data);

    if (isset($result->success)) {
      if (empty($result->data->pages)){
        $form['#prefix'] = '<p>' . t('The list of pages is empty. Add a page on Instapage first.') . '</p>';
      }
      else {
        $form['info']['#markup'] = '<p>' . t('This is the list of all pages on Instapage. Type the path on which you want to show Instapage page.') . '</p>';
        $form['info']['#markup'] .= "<p>" . t("If you don't want to submit certain page, leave the field empty.") . '</p>';
        $form['info']['#markup'] .= '<p>' . t('If you want to show Instapage as a front page, choose a path (for example: front) and then change default front page path <a href="@link">here</a>.', array('@link' => Url::fromRoute('system.site_information_settings'))) . '</p>';
        $config_pages = $this->config('instapage.pages');
        $pages = $config_pages->get('instapage_pages');
        $form['instapage_pages'] = array(
          '#tree' => TRUE
        );
        foreach ($result->data->pages as $page) {
          $form['instapage_pages'][$page->id] = array(
            '#type' => 'textfield',
            '#title' => $page->title,
            '#field_prefix' => $base_url . '/',
            '#default_value' => isset($pages[$page->id]) ? $pages[$page->id] : '',
          );
        }
        $form['actions']['#type'] = 'actions';
        $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Save'));
      }
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Save'));

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('instapage_pages')) {
      $paths = array();
      foreach ($form_state->getValue('instapage_pages') as $instapage_id => $path) {
        $error_field_name = 'instapage_pages][' . $instapage_id;
        if (in_array($path, $paths)) {
          if ($path != ''){
            $form_state->setErrorByName($error_field_name, t('Path @path is not unique on the list.', array('@path' => $path)));
          }
        }
        $paths[] = $path;
      }
    }
    else {
      $form_state->setErrorByName('form', t('Values are missing.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    global $base_url;
    $pages = array();

    $config_settings = $this->config('instapage.settings');
    $instapage_user_id = $config_settings->get('instapage_user_id');
    $plugin_hash = $config_settings->get('instapage_plugin_hash');

    $config_pages = $this->config('instapage.pages');
    $pages_list = $config_pages->get('instapage_pages');

    foreach ($form_state->getValue('instapage_pages') as $instapage_id => $path) {
      if ($path != '') {
        $pages[$instapage_id] = $path;

        // Let Instapage know about the path.
        $url = $base_url . '/' . $path;

        $data = array(
          'user_id' => $instapage_user_id,
          'plugin_hash' => $plugin_hash,
          'page_id' => $instapage_id,
          'url' => str_replace(array('http://', 'https://'), '', $url)
        );

        instapage_api_call('update-page', $data);
        unset($pages_list[$instapage_id]);
      }
    }

    foreach ($pages_list as $instapage_id => $path) {
      // Reset paths on Instapage that were removed.
      $data = array(
        'user_id' => $instapage_user_id,
        'plugin_hash' => $plugin_hash,
        'page_id' => $instapage_id,
        'url' => 'app.instapage.com/server/view-by-id/' . $instapage_id
      );
      instapage_api_call('update-page', $data);
    }

    $config_pages->set('instapage_pages', $pages);
    $config_pages->save();

    drupal_set_message(t('All pages have been saved.'));

    // Rebuild routing cache, to get all paths functional.
    \Drupal::service("router.builder")->rebuild();
  }

}
